@@ -69,7 +69,7 @@ import org.apache.tomcat.util.res.StringManager;
  * requests.  Requests of any other type will simply be passed through.
  *
  * @author Craig R. McClanahan
- * @version $Id: AuthenticatorBase.java 1126273 2011-05-22 22:54:15Z markt $
+ * @version $Id: AuthenticatorBase.java 1493918 2013-06-17 20:17:03Z markt $
  */
 
 
@@ -204,6 +204,16 @@ public abstract class AuthenticatorBase extends ValveBase
     // ------------------------------------------------------------- Properties
 
 
+    public boolean getAlwaysUseSession() {
+        return alwaysUseSession;
+    }
+
+
+    public void setAlwaysUseSession(boolean alwaysUseSession) {
+        this.alwaysUseSession = alwaysUseSession;
+    }
+
+
     /**
      * Return the cache authenticated Principals flag.
      */
@@ -444,6 +454,36 @@ public abstract class AuthenticatorBase extends ValveBase
             }
         }
 
+        // Special handling for form-based logins to deal with the case where
+        // a resource is protected for some HTTP methods but not protected for
+        // GET which is used after authentication when redirecting to the
+        // protected resource.
+        // TODO: This is similar to the FormAuthenticator.matchRequest() logic
+        //       Is there a way to remove the duplication?
+        Session session = request.getSessionInternal(false);
+        if (session != null) {
+            SavedRequest savedRequest =
+                    (SavedRequest) session.getNote(Constants.FORM_REQUEST_NOTE);
+            if (savedRequest != null) {
+                String decodedRequestURI = request.getDecodedRequestURI();
+                if (decodedRequestURI != null &&
+                        decodedRequestURI.equals(
+                                savedRequest.getDecodedRequestURI())) {
+                    if (!authenticate(request, response)) {
+                        if (log.isDebugEnabled()) {
+                            log.debug(" Failed authenticate() test");
+                        }
+                        /*
+                         * ASSERT: Authenticator already set the appropriate
+                         * HTTP status code, so we do not have to do anything
+                         * special
+                         */
+                        return;
+                    }
+                }
+            }
+        }
+
         // The Servlet may specify security constraints through annotations.
         // Ensure that they have been processed before constraints are checked
         Wrapper wrapper = (Wrapper) request.getMappingData().wrapper;
@@ -466,7 +506,6 @@ public abstract class AuthenticatorBase extends ValveBase
         // Make sure that constrained resources are not cached by web proxies
         // or browsers as caching can provide a security hole
         if (constraints != null && disableProxyCaching && 
-            !request.isSecure() &&
             !"POST".equalsIgnoreCase(request.getMethod())) {
             if (securePagesWithPragma) {
                 // Note: These can cause problems with downloading files with IE
@@ -516,13 +555,13 @@ public abstract class AuthenticatorBase extends ValveBase
             }
         }
 
-        if (!authRequired) {
+        if (!authRequired && context.getPreemptiveAuthentication()) {
             authRequired =
                 request.getCoyoteRequest().getMimeHeaders().getValue(
                         "authorization") != null;
         }
 
-        if (!authRequired) {
+        if (!authRequired && context.getPreemptiveAuthentication()) {
             X509Certificate[] certs = (X509Certificate[]) request.getAttribute(
                     Globals.CERTIFICATES_ATTR);
             authRequired = certs != null && certs.length > 0;
@@ -593,6 +632,27 @@ public abstract class AuthenticatorBase extends ValveBase
     }
 
 
+    /**
+     * Authenticate the user making this request, based on the login
+     * configuration of the {@link Context} with which this Authenticator is
+     * associated.  Return <code>true</code> if any specified constraint has
+     * been satisfied, or <code>false</code> if we have created a response
+     * challenge already.
+     *
+     * @param request Request we are processing
+     * @param response Response we are populating
+     *
+     * @exception IOException if an input/output error occurs
+     */
+    @Override
+    public boolean authenticate(Request request, HttpServletResponse response)
+            throws IOException {
+        if (context == null || context.getLoginConfig() == null) {
+            return true;
+        }
+        return authenticate(request, response, context.getLoginConfig());
+    }
+
     /**
      * Authenticate the user making this request, based on the specified
      * login configuration.  Return <code>true</code> if any specified
@@ -745,6 +805,7 @@ public abstract class AuthenticatorBase extends ValveBase
             if (principal == null) {
                 // Registering a programmatic logout
                 sso.deregister(ssoId);
+                request.removeNote(Constants.REQ_SSOID_NOTE);
                 return;
             } else {
                 // Update the SSO session with the latest authentication data
