@@ -78,7 +78,7 @@ import org.apache.tomcat.util.modeler.Util;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: StandardWrapper.java 1412576 2012-11-22 15:02:31Z markt $
+ * @version $Id: StandardWrapper.java 1476846 2013-04-28 22:04:06Z markt $
  */
 @SuppressWarnings("deprecation") // SingleThreadModel
 public class StandardWrapper extends ContainerBase
@@ -588,13 +588,20 @@ public class StandardWrapper extends ContainerBase
         
         // The logic to determine this safely is more complex than one might
         // expect. allocate() already has the necessary logic so re-use it.
+        // Make sure the Servlet is loaded with the right class loader
+        ClassLoader old = Thread.currentThread().getContextClassLoader();
+        ClassLoader webappClassLoader =
+                ((Context) getParent()).getLoader().getClassLoader();
         try {
+            Thread.currentThread().setContextClassLoader(webappClassLoader);
             Servlet s = allocate();
             deallocate(s);
         } catch (Throwable t) {
             ExceptionUtils.handleThrowable(t);
+        } finally {
+            Thread.currentThread().setContextClassLoader(old);
         }
-        return (singleThreadModel);
+        return singleThreadModel;
 
     }
 
@@ -1206,7 +1213,6 @@ public class StandardWrapper extends ContainerBase
 
     /**
      * {@inheritDoc}
-     * @throws ClassNotFoundException 
      */
     @Override
     public void servletSecurityAnnotationScan() throws ServletException {
@@ -1256,13 +1262,20 @@ public class StandardWrapper extends ContainerBase
                                               servlet);
 
             if( Globals.IS_SECURITY_ENABLED) {
-
-                Object[] args = new Object[]{(facade)};
-                SecurityUtil.doAsPrivilege("init",
-                                           servlet,
-                                           classType,
-                                           args);
-                args = null;
+                boolean success = false;
+                try {
+                    Object[] args = new Object[] { facade };
+                    SecurityUtil.doAsPrivilege("init",
+                                               servlet,
+                                               classType,
+                                               args);
+                    success = true;
+                } finally {
+                    if (!success) {
+                        // destroy() will not be called, thus clear the reference now
+                        SecurityUtil.remove(servlet);
+                    }
+                }
             } else {
                 servlet.init(facade);
             }
@@ -1435,7 +1448,8 @@ public class StandardWrapper extends ContainerBase
             while ((nRetries < 21) && (countAllocated.get() > 0)) {
                 if ((nRetries % 10) == 0) {
                     log.info(sm.getString("standardWrapper.waiting",
-                                          countAllocated.toString()));
+                                          countAllocated.toString(),
+                                          getName()));
                 }
                 try {
                     Thread.sleep(delay);
@@ -1458,9 +1472,12 @@ public class StandardWrapper extends ContainerBase
                   (InstanceEvent.BEFORE_DESTROY_EVENT, instance);
     
                 if( Globals.IS_SECURITY_ENABLED) {
-                    SecurityUtil.doAsPrivilege("destroy",
-                                               instance);
-                    SecurityUtil.remove(instance);                           
+                    try {
+                        SecurityUtil.doAsPrivilege("destroy",
+                                                   instance);
+                    } finally {
+                        SecurityUtil.remove(instance);
+                    }
                 } else {
                     instance.destroy();
                 }
@@ -1513,8 +1530,11 @@ public class StandardWrapper extends ContainerBase
                 while (!instancePool.isEmpty()) {
                     Servlet s = instancePool.pop();
                     if (Globals.IS_SECURITY_ENABLED) {
-                        SecurityUtil.doAsPrivilege("destroy", s);
-                        SecurityUtil.remove(instance);                           
+                        try {
+                            SecurityUtil.doAsPrivilege("destroy", s);
+                        } finally {
+                            SecurityUtil.remove(s);
+                        }
                     } else {
                         s.destroy();
                     }
