@@ -820,13 +820,13 @@ public class RemoteIpFilter implements Filter {
             }
             if (requestAttributesEnabled) {
                 request.setAttribute(AccessLog.REMOTE_ADDR_ATTRIBUTE,
-                        request.getRemoteAddr());
+                        xRequest.getRemoteAddr());
                 request.setAttribute(AccessLog.REMOTE_HOST_ATTRIBUTE,
-                        request.getRemoteHost());
+                        xRequest.getRemoteHost());
                 request.setAttribute(AccessLog.PROTOCOL_ATTRIBUTE,
-                        request.getProtocol());
+                        xRequest.getProtocol());
                 request.setAttribute(AccessLog.SERVER_PORT_ATTRIBUTE,
-                        Integer.valueOf(request.getServerPort()));
+                        Integer.valueOf(xRequest.getServerPort()));
             }
             chain.doFilter(xRequest, response);
         } else {
@@ -1104,10 +1104,10 @@ public class RemoteIpFilter implements Filter {
      * 
      * The attributes set are:
      * <ul>
-     * <li>org.apache.catalina.RemoteAddr</li>
-     * <li>org.apache.catalina.RemoteHost</li>
-     * <li>org.apache.catalina.Protocol</li>
-     * <li>org.apache.catalina.ServerPost</li>
+     * <li>org.apache.catalina.AccessLog.RemoteAddr</li>
+     * <li>org.apache.catalina.AccessLog.RemoteHost</li>
+     * <li>org.apache.catalina.AccessLog.Protocol</li>
+     * <li>org.apache.catalina.AccessLog.ServerPort</li>
      * </ul>
      * 
      * @param requestAttributesEnabled  <code>true</code> causes the attributes
