@@ -23,15 +23,13 @@ import java.io.IOException;
 import java.io.ObjectInput;
 import java.io.ObjectOutput;
 import java.io.Serializable;
-
 import java.security.Principal;
-
 import java.util.Arrays;
 import java.util.List;
 
 import org.apache.catalina.Realm;
 import org.apache.catalina.realm.GenericPrincipal;
-import org.apache.catalina.util.StringManager;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -40,17 +38,19 @@ import org.apache.catalina.util.StringManager;
  * The GenericPrincipal does NOT implement serializable and I didn't want to
  * change that implementation hence I implemented this one instead.
  * @author Filip Hanik
- * @version $Id: SerializablePrincipal.java 939514 2010-04-29 23:55:14Z kkolinko $
+ * @version $Id: SerializablePrincipal.java 1186071 2011-10-19 10:21:28Z markt $
  */
 public class SerializablePrincipal  implements java.io.Serializable {
 
-    protected static org.apache.juli.logging.Log log =
+    private static final long serialVersionUID = 1L;
+
+    private static final org.apache.juli.logging.Log log =
         org.apache.juli.logging.LogFactory.getLog(SerializablePrincipal.class);
     
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
     // ----------------------------------------------------------- Constructors
@@ -64,13 +64,12 @@ public class SerializablePrincipal  implements java.io.Serializable {
      * Construct a new Principal, associated with the specified Realm, for the
      * specified username and password.
      *
-     * @param realm The Realm that owns this Principal
      * @param name The username of the user represented by this Principal
      * @param password Credentials used to authenticate this user
      */
-    public SerializablePrincipal(Realm realm, String name, String password) {
+    public SerializablePrincipal(String name, String password) {
 
-        this(realm, name, password, null);
+        this(name, password, null);
 
     }
 
@@ -80,14 +79,13 @@ public class SerializablePrincipal  implements java.io.Serializable {
      * specified username and password, with the specified role names
      * (as Strings).
      *
-     * @param realm The Realm that owns this principal
      * @param name The username of the user represented by this Principal
      * @param password Credentials used to authenticate this user
      * @param roles List of roles (must be Strings) possessed by this user
      */
-    public SerializablePrincipal(Realm realm, String name, String password,
+    public SerializablePrincipal(String name, String password,
                             List<String> roles) {
-        this(realm, name, password, roles, null);
+        this(name, password, roles, null);
     }
 
     
@@ -96,23 +94,21 @@ public class SerializablePrincipal  implements java.io.Serializable {
      * specified username and password, with the specified role names
      * (as Strings).
      *
-     * @param realm The Realm that owns this principal
      * @param name The username of the user represented by this Principal
      * @param password Credentials used to authenticate this user
      * @param roles List of roles (must be Strings) possessed by this user
      * @param userPrincipal The user principal to be exposed to applications
      */
-    public SerializablePrincipal(Realm realm, String name, String password,
+    public SerializablePrincipal(String name, String password,
                             List<String> roles, Principal userPrincipal) {
 
         super();
-        this.realm = realm;
         this.name = name;
         this.password = password;
         if (roles != null) {
             this.roles = new String[roles.size()];
             this.roles = roles.toArray(this.roles);
-            if (this.roles.length > 0)
+            if (this.roles.length > 1)
                 Arrays.sort(this.roles);
         }
         if (userPrincipal instanceof Serializable) {
@@ -159,8 +155,6 @@ public class SerializablePrincipal  implements java.io.Serializable {
     }
 
 
-
-
     /**
      * The set of roles associated with this user.
      */
@@ -179,15 +173,14 @@ public class SerializablePrincipal  implements java.io.Serializable {
     // --------------------------------------------------------- Public Methods
 
 
-
-
     /**
      * Return a String representation of this object, which exposes only
      * information that should be public.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("SerializablePrincipal[");
+        StringBuilder sb = new StringBuilder("SerializablePrincipal[");
         sb.append(this.name);
         sb.append("]");
         return (sb.toString());
@@ -197,21 +190,20 @@ public class SerializablePrincipal  implements java.io.Serializable {
     public static SerializablePrincipal createPrincipal(GenericPrincipal principal)
     {
         if ( principal==null) return null;
-        return new SerializablePrincipal(principal.getRealm(),
-                                         principal.getName(),
+        return new SerializablePrincipal(principal.getName(),
                                          principal.getPassword(),
                                          principal.getRoles()!=null?Arrays.asList(principal.getRoles()):null,
                                          principal.getUserPrincipal()!=principal?principal.getUserPrincipal():null);
     }
 
-    public GenericPrincipal getPrincipal( Realm realm )
+    public GenericPrincipal getPrincipal()
     {
-        return new GenericPrincipal(realm, name, password,
+        return new GenericPrincipal(name, password,
                 getRoles()!=null?Arrays.asList(getRoles()):null,
                 userPrincipal);
     }
     
-    public static GenericPrincipal readPrincipal(ObjectInput in, Realm realm)
+    public static GenericPrincipal readPrincipal(ObjectInput in)
             throws IOException, ClassNotFoundException {
         String name = in.readUTF();
         boolean hasPwd = in.readBoolean();
@@ -231,7 +223,7 @@ public class SerializablePrincipal  implements java.io.Serializable {
                 throw e;
             }
         }
-        return new GenericPrincipal(realm,name,pwd,Arrays.asList(roles),
+        return new GenericPrincipal(name,pwd,Arrays.asList(roles),
                 userPrincipal);
     }
     
