@@ -47,7 +47,7 @@ import org.apache.tomcat.util.ExceptionUtils;
 * @author Craig R. McClanahan
 * @author Carson McDonald
 * @author Ignacio Ortega
-* @version $Id: JDBCRealm.java 1348499 2012-06-09 20:37:14Z markt $
+* @version $Id: JDBCRealm.java 1495202 2013-06-20 21:38:59Z markt $
 */
 
 public class JDBCRealm
@@ -546,48 +546,42 @@ public class JDBCRealm
         // connection may try to be opened again. On normal conditions (including
         // invalid login - the above is only used once.
         int numberOfTries = 2;
-        while (numberOfTries>0) {
+        while (numberOfTries > 0) {
             try {
-                
                 // Ensure that we have an open database connection
                 open();
                 
-                try {
-                    stmt = credentials(dbConnection, username);
-                    rs = stmt.executeQuery();
+                stmt = credentials(dbConnection, username);
+                rs = stmt.executeQuery();
+                dbConnection.commit();
                     
-                    if (rs.next()) {
-                        dbCredentials = rs.getString(1);
-                    }
-                    rs.close();
-                    rs = null;
-                    if (dbCredentials == null) {
-                        return (null);
-                    }
+                if (rs.next()) {
+                    dbCredentials = rs.getString(1);
+                }
                     
+                if (dbCredentials != null) {
                     dbCredentials = dbCredentials.trim();
-                    return dbCredentials;
-                    
-                } finally {
-                    if (rs!=null) {
-                        try {
-                            rs.close();
-                        } catch(SQLException e) {
-                            containerLog.warn(sm.getString("jdbcRealm.abnormalCloseResultSet"));
-                        }
-                    }
-                    dbConnection.commit();
                 }
                 
+                return dbCredentials;
+
             } catch (SQLException e) {
-                
                 // Log the problem for posterity
                 containerLog.error(sm.getString("jdbcRealm.exception"), e);
-                
-                // Close the connection so that it gets reopened next time
-                if (dbConnection != null)
-                    close(dbConnection);
-                
+            } finally {
+                if (rs != null) {
+                    try {
+                        rs.close();
+                    } catch(SQLException e) {
+                        containerLog.warn(sm.getString(
+                                "jdbcRealm.abnormalCloseResultSet"));
+                    }
+                }
+            }
+
+            // Close the connection so that it gets reopened next time
+            if (dbConnection != null) {
+                close(dbConnection);
             }
             
             numberOfTries--;
@@ -728,7 +722,10 @@ public class JDBCRealm
      * Release our use of this connection so that it can be recycled.
      *
      * @param dbConnection The connection to be released
+     *
+     * @deprecated  Unused
      */
+    @Deprecated
     protected void release(Connection dbConnection) {
 
         // NO-OP since we are not pooling anything
