@@ -20,8 +20,13 @@ package org.apache.catalina.startup;
 
 
 import java.io.File;
+import java.io.IOException;
 import java.net.URL;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Locale;
 import java.util.Set;
 
 import org.apache.catalina.loader.StandardClassLoader;
@@ -45,19 +50,14 @@ import org.apache.juli.logging.LogFactory;
  * </ul>
  *
  * @author Craig R. McClanahan
- * @version $Id: ClassLoaderFactory.java 939353 2010-04-29 15:50:43Z kkolinko $
+ * @version $Id: ClassLoaderFactory.java 1076059 2011-03-01 23:59:36Z kkolinko $
  */
 
 public final class ClassLoaderFactory {
 
 
-    private static Log log = LogFactory.getLog(ClassLoaderFactory.class);
-
-    protected static final Integer IS_DIR = new Integer(0);
-    protected static final Integer IS_JAR = new Integer(1);
-    protected static final Integer IS_GLOB = new Integer(2);
-    protected static final Integer IS_URL = new Integer(3);
-
+    private static final Log log = LogFactory.getLog(ClassLoaderFactory.class);
+    
     // --------------------------------------------------------- Public Methods
 
 
@@ -78,35 +78,7 @@ public final class ClassLoaderFactory {
      */
     public static ClassLoader createClassLoader(File unpacked[],
                                                 File packed[],
-                                                ClassLoader parent)
-        throws Exception {
-        return createClassLoader(unpacked, packed, null, parent);
-    }
-
-
-    /**
-     * Create and return a new class loader, based on the configuration
-     * defaults and the specified directory paths:
-     *
-     * @param unpacked Array of pathnames to unpacked directories that should
-     *  be added to the repositories of the class loader, or <code>null</code> 
-     * for no unpacked directories to be considered
-     * @param packed Array of pathnames to directories containing JAR files
-     *  that should be added to the repositories of the class loader, 
-     * or <code>null</code> for no directories of JAR files to be considered
-     * @param urls Array of URLs to remote repositories, designing either JAR 
-     *  resources or uncompressed directories that should be added to 
-     *  the repositories of the class loader, or <code>null</code> for no 
-     *  directories of JAR files to be considered
-     * @param parent Parent class loader for the new class loader, or
-     *  <code>null</code> for the system class loader.
-     *
-     * @exception Exception if an error occurs constructing the class loader
-     */
-    public static ClassLoader createClassLoader(File unpacked[],
-                                                File packed[],
-                                                URL urls[],
-                                                ClassLoader parent)
+                                                final ClassLoader parent)
         throws Exception {
 
         if (log.isDebugEnabled())
@@ -138,7 +110,7 @@ public final class ClassLoaderFactory {
                     continue;
                 String filenames[] = directory.list();
                 for (int j = 0; j < filenames.length; j++) {
-                    String filename = filenames[j].toLowerCase();
+                    String filename = filenames[j].toLowerCase(Locale.ENGLISH);
                     if (!filename.endsWith(".jar"))
                         continue;
                     File file = new File(directory, filenames[j]);
@@ -151,14 +123,17 @@ public final class ClassLoaderFactory {
         }
 
         // Construct the class loader itself
-        URL[] array = set.toArray(new URL[set.size()]);
-        StandardClassLoader classLoader = null;
-        if (parent == null)
-            classLoader = new StandardClassLoader(array);
-        else
-            classLoader = new StandardClassLoader(array, parent);
-        return (classLoader);
-
+        final URL[] array = set.toArray(new URL[set.size()]);
+        return AccessController.doPrivileged(
+                new PrivilegedAction<StandardClassLoader>() {
+                    @Override
+                    public StandardClassLoader run() {
+                        if (parent == null)
+                            return new StandardClassLoader(array);
+                        else
+                            return new StandardClassLoader(array, parent);
+                    }
+                });
     }
 
 
@@ -166,20 +141,16 @@ public final class ClassLoaderFactory {
      * Create and return a new class loader, based on the configuration
      * defaults and the specified directory paths:
      *
-     * @param locations Array of strings containing class directories, jar files,
-     *  jar directories or URLS that should be added to the repositories of
-     *  the class loader. The type is given by the member of param types.
-     * @param types Array of types for the members of param locations.
-     *  Possible values are IS_DIR (class directory), IS_JAR (single jar file),
-     *  IS_GLOB (directory of jar files) and IS_URL (URL).
+     * @param repositories List of class directories, jar files, jar directories
+     *                     or URLS that should be added to the repositories of
+     *                     the class loader.
      * @param parent Parent class loader for the new class loader, or
      *  <code>null</code> for the system class loader.
      *
      * @exception Exception if an error occurs constructing the class loader
      */
-    public static ClassLoader createClassLoader(String locations[],
-                                                Integer types[],
-                                                ClassLoader parent)
+    public static ClassLoader createClassLoader(List<Repository> repositories,
+                                                final ClassLoader parent)
         throws Exception {
 
         if (log.isDebugEnabled())
@@ -188,50 +159,52 @@ public final class ClassLoaderFactory {
         // Construct the "class path" for this class loader
         Set<URL> set = new LinkedHashSet<URL>();
 
-        if (locations != null && types != null && locations.length == types.length) {
-            for (int i = 0; i < locations.length; i++)  {
-                String location = locations[i];
-                if ( types[i] == IS_URL ) {
-                    URL url = new URL(location);
+        if (repositories != null) {
+            for (Repository repository : repositories)  {
+                if (repository.getType() == RepositoryType.URL) {
+                    URL url = new URL(repository.getLocation());
                     if (log.isDebugEnabled())
                         log.debug("  Including URL " + url);
                     set.add(url);
-                } else if ( types[i] == IS_DIR ) {
-                    File directory = new File(location);
-                    directory = new File(directory.getCanonicalPath());
-                    if (!directory.exists() || !directory.isDirectory() ||
-                        !directory.canRead())
-                         continue;
+                } else if (repository.getType() == RepositoryType.DIR) {
+                    File directory = new File(repository.getLocation());
+                    directory = directory.getCanonicalFile();
+                    if (!validateFile(directory, RepositoryType.DIR)) {
+                        continue;
+                    }
                     URL url = directory.toURI().toURL();
                     if (log.isDebugEnabled())
                         log.debug("  Including directory " + url);
                     set.add(url);
-                } else if ( types[i] == IS_JAR ) {
-                    File file=new File(location);
-                    file = new File(file.getCanonicalPath());
-                    if (!file.exists() || !file.canRead())
+                } else if (repository.getType() == RepositoryType.JAR) {
+                    File file=new File(repository.getLocation());
+                    file = file.getCanonicalFile();
+                    if (!validateFile(file, RepositoryType.JAR)) {
                         continue;
+                    }
                     URL url = file.toURI().toURL();
                     if (log.isDebugEnabled())
                         log.debug("  Including jar file " + url);
                     set.add(url);
-                } else if ( types[i] == IS_GLOB ) {
-                    File directory=new File(location);
-                    if (!directory.exists() || !directory.isDirectory() ||
-                        !directory.canRead())
+                } else if (repository.getType() == RepositoryType.GLOB) {
+                    File directory=new File(repository.getLocation());
+                    directory = directory.getCanonicalFile();
+                    if (!validateFile(directory, RepositoryType.GLOB)) {
                         continue;
+                    }
                     if (log.isDebugEnabled())
                         log.debug("  Including directory glob "
                             + directory.getAbsolutePath());
                     String filenames[] = directory.list();
                     for (int j = 0; j < filenames.length; j++) {
-                        String filename = filenames[j].toLowerCase();
+                        String filename = filenames[j].toLowerCase(Locale.ENGLISH);
                         if (!filename.endsWith(".jar"))
                             continue;
                         File file = new File(directory, filenames[j]);
-                        file = new File(file.getCanonicalPath());
-                        if (!file.exists() || !file.canRead())
+                        file = file.getCanonicalFile();
+                        if (!validateFile(file, RepositoryType.JAR)) {
                             continue;
+                        }
                         if (log.isDebugEnabled())
                             log.debug("    Including glob jar file "
                                 + file.getAbsolutePath());
@@ -243,19 +216,84 @@ public final class ClassLoaderFactory {
         }
 
         // Construct the class loader itself
-        URL[] array = set.toArray(new URL[set.size()]);
+        final URL[] array = set.toArray(new URL[set.size()]);
         if (log.isDebugEnabled())
             for (int i = 0; i < array.length; i++) {
                 log.debug("  location " + i + " is " + array[i]);
             }
-        StandardClassLoader classLoader = null;
-        if (parent == null)
-            classLoader = new StandardClassLoader(array);
-        else
-            classLoader = new StandardClassLoader(array, parent);
-        return (classLoader);
 
+        return AccessController.doPrivileged(
+                new PrivilegedAction<StandardClassLoader>() {
+                    @Override
+                    public StandardClassLoader run() {
+                        if (parent == null)
+                            return new StandardClassLoader(array);
+                        else
+                            return new StandardClassLoader(array, parent);
+                    }
+                });
     }
 
+    private static boolean validateFile(File file,
+            RepositoryType type) throws IOException {
+        if (RepositoryType.DIR == type || RepositoryType.GLOB == type) {
+            if (!file.exists() || !file.isDirectory() || !file.canRead()) {
+                String msg = "Problem with directory [" + file +
+                        "], exists: [" + file.exists() +
+                        "], isDirectory: [" + file.isDirectory() +
+                        "], canRead: [" + file.canRead() + "]";
 
+                File home = new File (Bootstrap.getCatalinaHome());
+                home = home.getCanonicalFile();
+                File base = new File (Bootstrap.getCatalinaBase());
+                base = base.getCanonicalFile();
+                File defaultValue = new File(base, "lib");
+
+                // Existence of ${catalina.base}/lib directory is optional.
+                // Hide the warning if Tomcat runs with separate catalina.home
+                // and catalina.base and that directory is absent.
+                if (!home.getPath().equals(base.getPath())
+                        && file.getPath().equals(defaultValue.getPath())
+                        && !file.exists()) {
+                    log.debug(msg);
+                } else {
+                    log.warn(msg);
+                }
+                return false;
+            }
+        } else if (RepositoryType.JAR == type) {
+            if (!file.exists() || !file.canRead()) {
+                log.warn("Problem with JAR file [" + file +
+                        "], exists: [" + file.exists() +
+                        "], canRead: [" + file.canRead() + "]");
+                return false;
+            }
+        }
+        return true;
+    }
+
+    public static enum RepositoryType {
+        DIR,
+        GLOB,
+        JAR,
+        URL
+    }
+    
+    public static class Repository {
+        private String location;
+        private RepositoryType type;
+        
+        public Repository(String location, RepositoryType type) {
+            this.location = location;
+            this.type = type;
+        }
+        
+        public String getLocation() {
+            return location;
+        }
+        
+        public RepositoryType getType() {
+            return type;
+        }
+    }
 }
