@@ -40,9 +40,11 @@ import java.security.cert.CertificateFactory;
 import java.security.cert.CollectionCertStoreParameters;
 import java.security.cert.PKIXBuilderParameters;
 import java.security.cert.X509CertSelector;
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.List;
 import java.util.Locale;
-import java.util.Vector;
 
 import javax.net.ssl.CertPathTrustManagerParameters;
 import javax.net.ssl.KeyManager;
@@ -70,7 +72,7 @@ import org.apache.tomcat.util.res.StringManager;
  * JSSE.<br/>
  * keytool -genkey -alias tomcat -keyalg RSA</br>
  * Use "changeit" as password (this is the default we use).
- * 
+ *
  * @author Harish Prabandham
  * @author Costin Manolache
  * @author Stefan Freyr Stefansson
@@ -87,6 +89,9 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
 
     private static final boolean RFC_5746_SUPPORTED;
 
+    private static final String[] DEFAULT_SERVER_PROTOCOLS;
+    private static final String[] DEAFULT_SERVER_CIPHER_SUITES;
+
     // Defaults - made public where re-used
     private static final String defaultProtocol = "TLS";
     private static final String defaultKeystoreType = "JKS";
@@ -96,27 +101,44 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
     private static final int defaultSessionTimeout = 86400;
     private static final String ALLOW_ALL_SUPPORTED_CIPHERS = "ALL";
     public static final String DEFAULT_KEY_PASS = "changeit";
-    
+
     static {
         boolean result = false;
         SSLContext context;
+        String[] ciphers = null;
+        String[] protocols = null;
         try {
             context = SSLContext.getInstance("TLS");
             context.init(null, null, null);
             SSLServerSocketFactory ssf = context.getServerSocketFactory();
-            String ciphers[] = ssf.getSupportedCipherSuites();
-            for (String cipher : ciphers) {
+            String supportedCiphers[] = ssf.getSupportedCipherSuites();
+            for (String cipher : supportedCiphers) {
                 if ("TLS_EMPTY_RENEGOTIATION_INFO_SCSV".equals(cipher)) {
                     result = true;
                     break;
                 }
             }
+
+            // There is no API to obtain the default server protocols and cipher
+            // suites. Having inspected the OpenJDK code there the same results
+            // can be achieved via the standard API but there is no guarantee
+            // that every JVM implementation determines the defaults the same
+            // way. Therefore the defaults are determined by creating a server
+            // socket and requested the configured values.
+
+            SSLServerSocket socket = (SSLServerSocket) ssf.createServerSocket();
+            ciphers = socket.getEnabledCipherSuites();
+            protocols = socket.getEnabledProtocols();
         } catch (NoSuchAlgorithmException e) {
             // Assume no RFC 5746 support
         } catch (KeyManagementException e) {
             // Assume no RFC 5746 support
+        } catch (IOException e) {
+            // Unable to determine default ciphers/protocols so use none
         }
         RFC_5746_SUPPORTED = result;
+        DEAFULT_SERVER_CIPHER_SUITES = ciphers;
+        DEFAULT_SERVER_PROTOCOLS = protocols;
     }
 
 
@@ -124,6 +146,7 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
 
     protected SSLServerSocketFactory sslProxy = null;
     protected String[] enabledCiphers;
+    protected String[] enabledProtocols;
     protected boolean allowUnsafeLegacyRenegotiation = false;
 
     /**
@@ -150,7 +173,7 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
         initServerSocket(socket);
         return socket;
     }
-    
+
     @Override
     public ServerSocket createSocket (int port, int backlog)
         throws IOException
@@ -160,19 +183,19 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
         initServerSocket(socket);
         return socket;
     }
-    
+
     @Override
     public ServerSocket createSocket (int port, int backlog,
                                       InetAddress ifAddress)
         throws IOException
-    {   
+    {
         init();
         ServerSocket socket = sslProxy.createServerSocket(port, backlog,
                                                           ifAddress);
         initServerSocket(socket);
         return socket;
     }
-    
+
     @Override
     public Socket acceptSocket(ServerSocket socket)
         throws IOException
@@ -185,7 +208,7 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
         }
         return asock;
     }
-    
+
     @Override
     public void handshake(Socket sock) throws IOException {
         // We do getSession instead of startHandshake() so we can call this multiple times
@@ -199,86 +222,48 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
         }
     }
 
-    /*
-     * Determines the SSL cipher suites to be enabled.
-     *
-     * @param requestedCiphers Comma-separated list of requested ciphers
-     * @param supportedCiphers Array of supported ciphers
-     *
-     * @return Array of SSL cipher suites to be enabled, or null if none of the
-     * requested ciphers are supported
-     */
-    protected String[] getEnabledCiphers(String requestedCiphers,
-                                         String[] supportedCiphers) {
+    @Override
+    public String[] getEnableableCiphers(SSLContext context) {
+        String requestedCiphersStr = endpoint.getCiphers();
 
-        String[] result = null;
-
-        if (ALLOW_ALL_SUPPORTED_CIPHERS.equals(requestedCiphers)) {
-            return supportedCiphers;
+        if (ALLOW_ALL_SUPPORTED_CIPHERS.equals(requestedCiphersStr)) {
+            return context.getSupportedSSLParameters().getCipherSuites();
+        }
+        if ((requestedCiphersStr == null)
+                || (requestedCiphersStr.trim().length() == 0)) {
+            return DEAFULT_SERVER_CIPHER_SUITES;
         }
 
-        if (requestedCiphers != null) {
-            Vector<String> vec = null;
-            String cipher = requestedCiphers;
-            int index = requestedCiphers.indexOf(',');
-            if (index != -1) {
-                int fromIndex = 0;
-                while (index != -1) {
-                    cipher =
-                        requestedCiphers.substring(fromIndex, index).trim();
-                    if (cipher.length() > 0) {
-                        /*
-                         * Check to see if the requested cipher is among the
-                         * supported ciphers, i.e., may be enabled
-                         */
-                        for (int i=0; supportedCiphers != null
-                                     && i<supportedCiphers.length; i++) {
-                            if (supportedCiphers[i].equals(cipher)) {
-                                if (vec == null) {
-                                    vec = new Vector<String>();
-                                }
-                                vec.addElement(cipher);
-                                break;
-                            }
-                        }
-                    }
-                    fromIndex = index+1;
-                    index = requestedCiphers.indexOf(',', fromIndex);
-                } // while
-                cipher = requestedCiphers.substring(fromIndex);
+        List<String> requestedCiphers = new ArrayList<String>();
+        for (String rc : requestedCiphersStr.split(",")) {
+            final String cipher = rc.trim();
+            if (cipher.length() > 0) {
+                requestedCiphers.add(cipher);
             }
+        }
+        if (requestedCiphers.isEmpty()) {
+            return DEAFULT_SERVER_CIPHER_SUITES;
+        }
+        List<String> ciphers = new ArrayList<String>(requestedCiphers);
+        ciphers.retainAll(Arrays.asList(context.getSupportedSSLParameters()
+                .getCipherSuites()));
 
-            if (cipher != null) {
-                cipher = cipher.trim();
-                if (cipher.length() > 0) {
-                    /*
-                     * Check to see if the requested cipher is among the
-                     * supported ciphers, i.e., may be enabled
-                     */
-                    for (int i=0; supportedCiphers != null
-                                 && i<supportedCiphers.length; i++) {
-                        if (supportedCiphers[i].equals(cipher)) {
-                            if (vec == null) {
-                                vec = new Vector<String>();
-                            }
-                            vec.addElement(cipher);
-                            break;
-                        }
-                    }
-                }
-            }           
-
-            if (vec != null) {
-                result = new String[vec.size()];
-                vec.copyInto(result);
+        if (ciphers.isEmpty()) {
+            log.warn(sm.getString("jsse.requested_ciphers_not_supported",
+                    requestedCiphersStr));
+        }
+        if (log.isDebugEnabled()) {
+            log.debug(sm.getString("jsse.enableable_ciphers", ciphers));
+            if (ciphers.size() != requestedCiphers.size()) {
+                List<String> skipped = new ArrayList<String>(requestedCiphers);
+                skipped.removeAll(ciphers);
+                log.debug(sm.getString("jsse.unsupported_ciphers", skipped));
             }
-        } else {
-            result = sslProxy.getDefaultCipherSuites();
         }
 
-        return result;
+        return ciphers.toArray(new String[ciphers.size()]);
     }
-     
+
     /*
      * Gets the SSL server's keystore password.
      */
@@ -399,10 +384,10 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
                 }
                 istream = new FileInputStream(keyStoreFile);
             }
-            
+
             char[] storePass = null;
             if (pass != null && !"".equals(pass)) {
-                storePass = pass.toCharArray(); 
+                storePass = pass.toCharArray();
             }
             ks.load(istream, storePass);
         } catch (FileNotFoundException fnfe) {
@@ -459,13 +444,12 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
             sslProxy = context.getServerSocketFactory();
 
             // Determine which cipher suites to enable
-            String requestedCiphers = endpoint.getCiphers();
-            enabledCiphers = getEnabledCiphers(requestedCiphers,
-                    sslProxy.getSupportedCipherSuites());
+            enabledCiphers = getEnableableCiphers(context);
+            enabledProtocols = getEnableableProtocols(context);
 
             allowUnsafeLegacyRenegotiation = "true".equals(
                     endpoint.getAllowUnsafeLegacyRenegotiation());
-            
+
             // Check the SSL config is OK
             checkConfig();
 
@@ -485,11 +469,11 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
             protocol = defaultProtocol;
         }
 
-        SSLContext context = SSLContext.getInstance(protocol); 
+        SSLContext context = SSLContext.getInstance(protocol);
 
         return context;
     }
-    
+
     @Override
     public KeyManager[] getKeyManagers() throws Exception {
         String keystoreType = endpoint.getKeystoreType();
@@ -518,7 +502,7 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
         if (truststoreType == null) {
             truststoreType = defaultKeystoreType;
         }
-        
+
         String algorithm = endpoint.getTruststoreAlgorithm();
         if (algorithm == null) {
             algorithm = TrustManagerFactory.getDefaultAlgorithm();
@@ -537,7 +521,7 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
         } else {
             sessionCacheSize = defaultSessionCacheSize;
         }
-        
+
         int sessionTimeout;
         if (endpoint.getSessionTimeout() != null) {
             sessionTimeout = Integer.parseInt(endpoint.getSessionTimeout());
@@ -596,7 +580,7 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
             String keystoreProvider, String algorithm)
         throws Exception {
         String crlf = endpoint.getCrlFile();
-        
+
         String className = endpoint.getTrustManagerClassName();
         if(className != null && className.length() > 0) {
              ClassLoader classLoader = getClass().getClassLoader();
@@ -608,10 +592,10 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
              Object trustManagerObject = clazz.newInstance();
              TrustManager trustManager = (TrustManager) trustManagerObject;
              return new TrustManager[]{ trustManager };
-        }    
+        }
 
         TrustManager[] tms = null;
-        
+
         KeyStore trustStore = getTrustStore(keystoreType, keystoreProvider);
         if (trustStore != null || endpoint.getTrustManagerClassName() != null) {
             if (crlf == null) {
@@ -630,21 +614,21 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
                 tms = tmf.getTrustManagers();
             }
         }
-        
+
         return tms;
     }
 
     /**
      * Return the initialization parameters for the TrustManager.
      * Currently, only the default <code>PKIX</code> is supported.
-     * 
+     *
      * @param algorithm The algorithm to get parameters for.
      * @param crlf The path to the CRL file.
      * @param trustStore The configured TrustStore.
      * @return The parameters including the CRLs and TrustStore.
      */
-    protected CertPathParameters getParameters(String algorithm, 
-                                                String crlf, 
+    protected CertPathParameters getParameters(String algorithm,
+                                                String crlf,
                                                 KeyStore trustStore)
         throws Exception {
         CertPathParameters params = null;
@@ -675,9 +659,9 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
 
     /**
      * Load the collection of CRLs.
-     * 
+     *
      */
-    protected Collection<? extends CRL> getCRLs(String crlf) 
+    protected Collection<? extends CRL> getCRLs(String crlf)
         throws IOException, CRLException, CertificateException {
 
         File crlFile = new File(crlf);
@@ -697,7 +681,7 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
             throw crle;
         } catch(CertificateException ce) {
             throw ce;
-        } finally { 
+        } finally {
             if(is != null) {
                 try{
                     is.close();
@@ -709,60 +693,32 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
         return crls;
     }
 
-    /**
-     * Set the SSL protocol variants to be enabled.
-     * @param socket the SSLServerSocket.
-     * @param protocols the protocols to use.
-     */
-    protected void setEnabledProtocols(SSLServerSocket socket,
-            String []protocols){
-        if (protocols != null) {
-            socket.setEnabledProtocols(protocols);
-        }
-    }
-
-    /**
-     * Determines the SSL protocol variants to be enabled.
-     *
-     * @param socket The socket to get supported list from.
-     * @param requestedProtocols Array of requested protocol names all of which
-     *                           must be non-null and non-zero length
-     *
-     * @return Array of SSL protocol variants to be enabled, or null if none of
-     * the requested protocol variants are supported
-     */
-    protected String[] getEnabledProtocols(SSLServerSocket socket,
-                                           String[] requestedProtocols){
-        String[] supportedProtocols = socket.getSupportedProtocols();
-
-        String[] enabledProtocols = null;
-
-        if (requestedProtocols != null && requestedProtocols.length > 0) {
-            Vector<String> vec = null;
-            for (String protocol : requestedProtocols) {
-                /*
-                 * Check to see if the requested protocol is among the supported
-                 * protocols, i.e., may be enabled
-                 */
-                for (int i=0; supportedProtocols != null &&
-                        i < supportedProtocols.length; i++) {
-                    if (supportedProtocols[i].equals(protocol)) {
-                        if (vec == null) {
-                            vec = new Vector<String>();
-                        }
-                        vec.addElement(protocol);
-                        break;
-                    }
-                }
-            }
-
-            if (vec != null) {
-                enabledProtocols = new String[vec.size()];
-                vec.copyInto(enabledProtocols);
-            }
+    @Override
+    public String[] getEnableableProtocols(SSLContext context) {
+        String[] requestedProtocols = endpoint.getSslEnabledProtocolsArray();
+        if ((requestedProtocols == null) || (requestedProtocols.length == 0)) {
+            return DEFAULT_SERVER_PROTOCOLS;
         }
 
-        return enabledProtocols;
+        List<String> protocols = new ArrayList<String>(
+                Arrays.asList(requestedProtocols));
+        protocols.retainAll(Arrays.asList(context.getSupportedSSLParameters()
+                .getProtocols()));
+
+        if (protocols.isEmpty()) {
+            log.warn(sm.getString("jsse.requested_protocols_not_supported",
+                    Arrays.asList(requestedProtocols)));
+        }
+        if (log.isDebugEnabled()) {
+            log.debug(sm.getString("jsse.enableable_protocols", protocols));
+            if (protocols.size() != requestedProtocols.length) {
+                List<String> skipped = new ArrayList<String>(
+                        Arrays.asList(requestedProtocols));
+                skipped.removeAll(protocols);
+                log.debug(sm.getString("jsse.unsupported_protocols", skipped));
+            }
+        }
+        return protocols.toArray(new String[protocols.size()]);
     }
 
     /**
@@ -787,14 +743,9 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
 
         SSLServerSocket socket = (SSLServerSocket) ssocket;
 
-        if (enabledCiphers != null) {
-            socket.setEnabledCipherSuites(enabledCiphers);
-        }
-
-        String[] requestedProtocols = endpoint.getSslEnabledProtocolsArray();
-        setEnabledProtocols(socket, getEnabledProtocols(socket, 
-                                                         requestedProtocols));
-
+        socket.setEnabledCipherSuites(enabledCiphers);
+        socket.setEnabledProtocols(enabledProtocols);
+        
         // we don't know if client auth is needed -
         // after parsing the request we may re-handshake
         configureClientAuth(socket);
@@ -812,7 +763,7 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
 
         try {
             // Set the timeout to 1ms as all we care about is if it throws an
-            // SSLException on accept. 
+            // SSLException on accept.
             socket.setSoTimeout(1);
 
             socket.accept();
@@ -840,6 +791,6 @@ public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
                 socket.close();
             }
         }
-        
+
     }
 }
