@@ -83,7 +83,6 @@ public class TestCsrfPreventionFilter extends TomcatBaseTest {
         ByteArrayInputStream bais =
             new ByteArrayInputStream(baos.toByteArray());
         ObjectInputStream ois = new ObjectInputStream(bais);
-        @SuppressWarnings("unchecked")
         LruCache<String> cache2 = (LruCache<String>) ois.readObject();
 
         cache2.add("key7");
@@ -103,7 +102,7 @@ public class TestCsrfPreventionFilter extends TomcatBaseTest {
         }
     }
 
-    private static class NonEncodingResponse extends TesterResponse {
+    private static class NonEncodingResponse extends TesterHttpServletResponse {
 
         @Override
         public String encodeRedirectURL(String url) {
